from decimal import Decimal
from django.db import models
from django.core.validators import MinValueValidator
from suppliers.models import Supplier
from colors.models import Color
from account.models import Account
import random
from gauge.models import Gauge

STATUS_CHOICES = [
    ('Available', 'Available'),
    ('Finished', 'Finished'),
    ('Blocked', 'Blocked'), 
]

GAUGE_CHOICES = [
    ('G30', 'G30'),
    ('G28', 'G28'),
]

FINISH_CHOICES = [
    ('matte', 'Matte'),
    ('glossy', 'Glossy'),
]

class Coil(models.Model):
    coil_number = models.CharField(max_length=50, unique=True)
    gauge = models.ForeignKey(Gauge, on_delete=models.CASCADE, blank=True, null=True)
    color = models.ForeignKey(Color, on_delete=models.CASCADE, blank=True, null=True)
    finish = models.CharField(max_length=20, choices=FINISH_CHOICES, default='matte')
    gross_weight = models.DecimalField(max_digits=10, decimal_places=3, validators=[MinValueValidator(0)])
    net_weight = models.DecimalField(max_digits=10, decimal_places=3, validators=[MinValueValidator(0)])
    meters = models.DecimalField(max_digits=10, decimal_places=2, validators=[MinValueValidator(0)])
    meters_used = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    rejects = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    waste_length = models.DecimalField(max_digits=10, decimal_places=2, default=0)
    status = models.CharField(max_length=20, choices=STATUS_CHOICES, default='Available')
    is_allocated = models.BooleanField(default=False)
    is_blocked = models.BooleanField(default=False)
    block_reason = models.TextField(default='', blank=True)
    supplier = models.ForeignKey(Supplier, on_delete=models.SET_NULL, null=True)
    added_by = models.ForeignKey(Account, on_delete=models.SET_NULL, null=True, blank=True)
    created_at = models.DateTimeField(auto_now_add=True, null=True, blank=True)
    updated_at = models.DateTimeField(auto_now=True, null=True, blank=True)

    def save(self, *args, **kwargs):
        self.original_meters = self.meters
        if self.is_blocked:
            self.status = 'Blocked'
        else:
            if Decimal(self.original_meters) < Decimal('5.0'):
                self.status = 'Finished'
            else:
                self.status = 'Available'
            meters_decimal = Decimal(str(self.meters))
            if Decimal(meters_decimal) < Decimal('5.0'):
                self.status = 'Finished'
            else:
                self.status = 'Available'
        super().save(*args, **kwargs)

    def __str__(self):
        return f"Coil {self.coil_number}"
